@@ -2,17 +2,18 @@
 
 #include <algorithm>
 #include <functional>
-#include <string> // in C++17 replace with <string_view>
+#include <string_view>
+#include <cctype>
 
 namespace detail {
 
 using uchar_type = const unsigned char; // we need to convert chars to unsigned chars because of std::tolower's requirements
-using str_it = std::string::const_iterator;
+using str_it = std::string_view::const_iterator;
 using comparator = bool (*)(uchar_type, uchar_type);
 using algorithm = bool (*)(str_it, str_it, str_it, str_it, comparator);
 
 template <typename Comparator>
-inline bool icase_for_each(const std::string& a, const std::string& b, algorithm alg)
+inline bool icase_for_each(std::string_view a, std::string_view b, algorithm alg)
 {
     return alg(a.begin(), a.end(), 
                b.begin(), b.end(),
@@ -23,7 +24,7 @@ inline bool icase_for_each(const std::string& a, const std::string& b, algorithm
 /**
  * @brief Case insensitive comparison of two strings.
  */ 
-inline bool icase_equals(const std::string& a, const std::string& b)
+inline bool icase_equals(std::string_view a, std::string_view b)
 {
      return detail::icase_for_each<std::equal_to<>>(a, b, std::equal);
 }
@@ -31,7 +32,7 @@ inline bool icase_equals(const std::string& a, const std::string& b)
 /**
  * @brief Case insensitive ordering of two strings.
  */ 
-inline bool icase_less_than(const std::string& a, const std::string& b)
+inline bool icase_less_than(std::string_view a, std::string_view b)
 {
      return detail::icase_for_each<std::less<>>(a, b, std::lexicographical_compare);
 }
