@@ -991,7 +991,7 @@ void thconfig::log_outcs(double decsyear, double deceyear) {
           firstdec = false;
         }
         if (this->get_outcs_mag_decl(double(yyy), dec)) {
-          thlog.printf("  %4d.1.1  %.4f\n", yyy, dec);
+          thlog.printf("  %4ld.1.1  %.4f\n", yyy, dec);
         }
       }
     }
