@@ -35,6 +35,7 @@
 #include "thparse.h"
 #include "thdate.h"
 #include "thdataleg.h"
+#include "thlayoutclr.h"
 #include <stdio.h>
 #include <map>
 
@@ -46,11 +47,11 @@ enum {
   TT_UNKNOWN_CMD,
   TT_DATAOBJECT_CMD, TT_DATASTATION_CMD, TT_2DDATAOBJECT_CMD,
   TT_SURVEY_CMD, TT_ENDSURVEY_CMD,
-  TT_GRADE_CMD, TT_LAYOUT_CMD,
+  TT_GRADE_CMD, TT_LAYOUT_CMD, TT_COMMENT_CMD,
   TT_SCRAP_CMD, TT_ENDSCRAP_CMD,
   TT_POINT_CMD, TT_LINE_CMD, TT_AREA_CMD,
   TT_JOIN_CMD, TT_MAP_CMD, TT_SURFACE_CMD,
-  TT_DATA_CMD, TT_IMPORT_CMD,
+  TT_DATA_CMD, TT_IMPORT_CMD, TT_SKETCH_CMD, TT_LOOKUP_CMD,
 };
 
 
@@ -62,6 +63,7 @@ static const thstok thtt_commands[] = {
   {"area", TT_AREA_CMD},
   {"centerline", TT_DATA_CMD},
   {"centreline", TT_DATA_CMD},
+  {"comment", TT_COMMENT_CMD},
   {"endscrap", TT_ENDSCRAP_CMD},
   {"endsurvey", TT_ENDSURVEY_CMD},
   {"grade", TT_GRADE_CMD},
@@ -69,9 +71,11 @@ static const thstok thtt_commands[] = {
   {"join", TT_JOIN_CMD},
   {"layout", TT_LAYOUT_CMD},
   {"line", TT_LINE_CMD},
+  {"lookup", TT_LOOKUP_CMD},
   {"map", TT_MAP_CMD},
   {"point", TT_POINT_CMD},
   {"scrap", TT_SCRAP_CMD},
+  {"sketch", TT_SKETCH_CMD},
   {"surface", TT_SURFACE_CMD},
   {"survey", TT_SURVEY_CMD},
 	{NULL, TT_UNKNOWN_CMD},
@@ -89,6 +93,8 @@ enum {
   TT_DATAOBJECT_AUTHOR = 1003,
   TT_DATAOBJECT_COPYRIGHT = 1004,
   TT_DATAOBJECT_STNS = 1005,
+  TT_DATAOBJECT_CS = 1006,
+  TT_DATAOBJECT_ATTR = 1007,
 };
 
 
@@ -97,8 +103,10 @@ enum {
  */
  
 static const thstok thtt_dataobject_opt[] = {
+  {"attr", TT_DATAOBJECT_ATTR},
   {"author", TT_DATAOBJECT_AUTHOR},
   {"copyright", TT_DATAOBJECT_COPYRIGHT},
+  {"cs", TT_DATAOBJECT_CS},
   {"id", TT_DATAOBJECT_NAME},
   {"station-names", TT_DATAOBJECT_STNS},
   {"title", TT_DATAOBJECT_TITLE},
@@ -106,7 +114,6 @@ static const thstok thtt_dataobject_opt[] = {
 };
 
 
-
 /**
  * Option description class.
  */
@@ -198,7 +205,7 @@ class thdataobject_copyright {
   
   public:
 
-  char * name;  ///< Author's name
+  const char * name;  ///< Author's name
   unsigned long rev;  ///< Object's revision.
   
   /**
@@ -212,7 +219,7 @@ class thdataobject_copyright {
    * Default constructor.
    */
    
-  thdataobject_copyright(char * nn, unsigned long rv) : name(nn), rev(rv) {}
+  thdataobject_copyright(const char * nn, unsigned long rv) : name(nn), rev(rv) {}
   
   
 };
@@ -239,7 +246,7 @@ class thdataobject {
 
   class thdatabase * db;
 
-  char * name,  ///< Object name.
+  const char * name,  ///< Object name.
     * title,  ///< Object title.
     * stnpref, ///< Station names prefix;
     * stnsuff; ///< Station names suffix;
@@ -247,10 +254,12 @@ class thdataobject {
   unsigned long id;  ///< Object identifier.
   
   bool selected,  ///< Whether object is selected.
-    tmp_bool;  ///< Temporary variable for some algorithms
+    tmp_bool = false;  ///< Temporary variable for some algorithms
+
+  thlayout_color selected_color; ///< Selection color.
     
   unsigned long selected_number,  ///< Number of selection.
-    tmp_ulong;  ///< Temporary variable for some algorithms
+    tmp_ulong = 0;  ///< Temporary variable for some algorithms
   
   thdataobject * nsptr,  ///< Next object in survey.
     * psptr;  ///< Previous object in survey.
@@ -266,6 +275,8 @@ class thdataobject {
   thdo_author_map_type author_map;  ///< Author map.
   thdo_copyright_map_type copyright_map;  ///< Copyright map.
 
+  int cs;  ///< Coordinate system.
+	thobjectsrc cs_source;  ///< Coordinate system source.
 
   public:
 
@@ -301,7 +312,7 @@ class thdataobject {
    * Return class name.
    */
    
-  virtual char * get_class_name() {return "thdataobject";};
+  virtual const char * get_class_name() {return "thdataobject";};
   
   
   /**
@@ -322,7 +333,7 @@ class thdataobject {
    * Return command end option.
    */
    
-  virtual char * get_cmd_end();
+  virtual const char * get_cmd_end();
   
   
   /**
@@ -343,21 +354,21 @@ class thdataobject {
    * Return command name.
    */
    
-  virtual char * get_cmd_name();
+  virtual const char * get_cmd_name();
   
   
   /**
    * Return option description.
    */
    
-  virtual thcmd_option_desc get_cmd_option_desc(char * opts);
+  virtual thcmd_option_desc get_cmd_option_desc(const char * opts);
   
   
   /**
    * Set command option.
    *
    * @param cod Command option description.
-   * @param args Option arguments arry.
+   * @param args Option arguments array.
    * @param argenc Arguments encoding.
    */
    
@@ -375,23 +386,14 @@ class thdataobject {
    * Return object name.
    */
    
-  virtual char * get_name();
+  virtual const char * get_name();
   
   
   /**
    * Return object title.
    */
    
-  char * get_title();
-  
-  
-  /**
-   * Delete this object.
-   *
-   * @warn Always use this methos instead of delete function.
-   */
-   
-  virtual void self_delete();
+  const char * get_title();
   
   
   /**
@@ -416,10 +418,10 @@ class thdataobject {
   
     
   /**
-   * Write object source to exception description.
+   * Get object source location.
    */
    
-  virtual void throw_source();
+  virtual std::string throw_source() const;
   
   
   /**
@@ -450,6 +452,33 @@ class thdataobject {
    */
  
   bool is_in_survey(thsurvey * psearch);
+
+
+  /**
+   * Read coordinates and estimate approximate location for PROJ conversions.
+   */
+
+  void read_cs(char * src_x, char * src_y, double & dst_x, double & dst_y, bool adj_bbox = true);
+
+
+  /**
+   * Convert coordinates according to current CS specification.
+   */
+
+  void convert_cs(int src_cs, double src_x, double src_y, double & dst_x, double & dst_y);
+
+  /**
+   * Convert all points in object.
+   */
+
+  virtual void convert_all_cs();
+
+  /**
+   * Parse object attributes.
+   */
+
+  void parse_attribute(char * name, char * value);
+
 };
 
 
