@@ -34,10 +34,16 @@
 #include "thdataleg.h"
 #include "thdb3d.h"
 #include "thdataleg.h"
+#include "thattr.h"
 #include <map>
 #include <vector>
 #include <list>
 
+enum {
+  TT_TEMPSTATION_NONE,
+  TT_TEMPSTATION_WALL,
+  TT_TEMPSTATION_FEATURE,
+};
 
 struct thdb1d_loop_leg {
   thdb1d_loop_leg * next_leg, * prev_leg;
@@ -49,27 +55,42 @@ struct thdb1d_loop {
   thdb1d_loop_leg * first_leg, * last_leg;
   class thdb1ds * from, * to;
   unsigned long nlegs;
+  unsigned long id;
   bool open;
-  double err_dx, err_dy, err_dz, err_length, src_length;
+  double err_dx, err_dy, err_dz, err_length, src_length, err;
+};
+
+struct thdb1d_traverse {
+  thdb1d_loop_leg * first_leg, * last_leg;
+  class thdb1ds * from, * to;
+  unsigned long nlegs;
+  unsigned long id;
+  double src_length, E, H, V;
 };
 
 class thdb1d_tree_node {
 
   public:
 
-  bool is_attached, is_fixed, xx_left, xx_touched;
+
+  bool is_attached, is_fixed;
+  int xx_left;
+  bool xx_touched, extendx_ok;
 
   thdb1d_tree_node * next_eq, * prev_eq;
   
   unsigned long id, uid, narrows;
   
-  double xx;
+  double xx, extendx, xx_ratio;
   
   class thdb1d_tree_arrow * first_arrow, * last_arrow, * back_arrow;
+  std::list<class thdb1d_tree_arrow * > back_buffer;
+  void push_back_arrow(class thdb1d_tree_arrow * arrow);
+  class thdb1d_tree_arrow * pop_back_arrow();
   
-  thdb1d_tree_node() : is_attached(false), is_fixed(false), xx_left(false), xx_touched(false),
+  thdb1d_tree_node() : is_attached(false), is_fixed(false), xx_left(1), xx_touched(false), extendx_ok(false),
     next_eq(NULL), prev_eq(NULL),
-    id(0), uid(0), narrows(0), xx(0.0),
+    id(0), uid(0), narrows(0), xx(0.0), extendx(0.0), xx_ratio(1.0),
     first_arrow(NULL), last_arrow(NULL), back_arrow(NULL) {}
   
 };
@@ -80,19 +101,21 @@ class thdb1d_tree_arrow {
 
   public:
 
-  bool is_discovery, is_reversed;
+  bool is_discovery, is_nosurvey, is_reversed;
   
   thdb1d_tree_node * start_node, * end_node;
   
   class thdb1dl * leg;
   
-  unsigned char extend;
+  unsigned extend;
+
+  std::list<unsigned long> extend_ignore_before;
   
   thdb1d_tree_arrow * negative;
   
   thdb1d_tree_arrow * next_arrow;
   
-  thdb1d_tree_arrow() : is_discovery(false), is_reversed(false),
+  thdb1d_tree_arrow() : is_discovery(false), is_nosurvey(false), is_reversed(false),
     start_node(NULL), end_node(NULL), 
     leg(NULL), extend(TT_EXTENDFLAG_NORMAL), negative(NULL), next_arrow(NULL) {}
   
@@ -119,55 +142,95 @@ class thdb1ds {
   
   double x, y, z;   // coordinates
   double xx;        // coordinate for extended elevation
+  double asl; // average cave shot length from this point
   
-  char * name,  ///< Station name.
+  const char * name,  ///< Station name.
     * comment;  ///< Station comment.
   
   class thsurvey * survey;  ///< Station survey.
+  class thdataobject * fixcontext;  ///< Fix context.
   bool tmpselect;
   
-  class thdata * data;  ///< Station data.
-  unsigned data_priority, ///< 0 - undefined, 1 - equate, 2 - leg, 3 - fix
-    data_slength;  ///< Survey specification length
+  std::list<class thdata *> data;  ///< Station data.
+  unsigned data_priority; ///< 0 - undefined, 1 - equate, 2 - leg, 3 - fix
   
-  unsigned char flags,  ///< Station flags.
-    mark,  ///< Mark type.
-    extend;  ///< Extend flags: normal, reverse, left, right, break
+  char temps;           ///< Station temporary status.
+  unsigned long flags;  ///< Station flags.
+  int mark,             ///< Mark type.
+    extend;             ///< Extend flags: normal, reverse, left, right, break
+  double extend_ratio;
   bool mark_station;
   
   bool adjusted, fixed;
   int placed;
   double sdx, sdy, sdz;
   
+  double explored;
+  
+  bool d3_parsed; ///< Whether splay shots have been parsed.
+  thdb3ddata d3_outline; ///< splay 3d outline.
+
+
   /**
    * Default constructor.
    */
    
-  thdb1ds() : uid(0), x(0), y(0), z(0), name(NULL), comment(NULL), survey(NULL), 
-    data(NULL), data_priority(0), data_slength(0), 
-    flags(TT_STATIONFLAG_NONE), mark(TT_DATAMARK_TEMP), extend(TT_EXTENDFLAG_NORMAL), 
-    adjusted(false), fixed(false), placed(0), sdx(0.0), sdy(0.0), sdz(0.0) {}
+  thdb1ds() : uid(0), x(0), y(0), z(0), xx(0), asl(0), name(NULL), comment(NULL), survey(NULL), fixcontext(NULL), tmpselect(false),
+    data_priority(0), temps(TT_TEMPSTATION_NONE),
+    flags(TT_STATIONFLAG_NONE), mark(TT_DATAMARK_TEMP), extend(TT_EXTENDFLAG_NORMAL), extend_ratio(thnan), mark_station(false),
+    adjusted(false), fixed(false), placed(0), sdx(0.0), sdy(0.0), sdz(0.0),
+    explored(thnan), d3_parsed(false) {}
   
 
   /**
    * Default constructor with 2 parameters.
    */
    
-  thdb1ds(char * n, class thsurvey * ps) : uid(0), x(0), y(0), z(0), name(n), 
-    comment(NULL), survey(ps), 
-    data(NULL), data_priority(0), data_slength(0), 
+  thdb1ds(const char * n, class thsurvey * ps) : uid(0), x(0), y(0), z(0), xx(0), asl(0), name(n), 
+    comment(NULL), survey(ps), fixcontext(NULL), tmpselect(false),
+    data_priority(0), temps(TT_TEMPSTATION_NONE),
     flags(TT_STATIONFLAG_NONE),
-    mark(TT_DATAMARK_TEMP), extend(TT_EXTENDFLAG_NORMAL), mark_station(false), 
-    adjusted(false), fixed(false), placed(0), sdx(0.0), sdy(0.0), sdz(0.0) {}
+    mark(TT_DATAMARK_TEMP), extend(TT_EXTENDFLAG_NORMAL), extend_ratio(thnan), mark_station(false),
+    adjusted(false), fixed(false), placed(0), sdx(0.0), sdy(0.0), sdz(0.0),
+    explored(thnan), d3_parsed(false) {}
     
   
   /**
    * Set parent data.
    */
   
-  void set_parent_data(class thdata * pd, unsigned pd_priority, unsigned pd_slength);
+  void set_parent_data(class thdata * pd, unsigned pd_priority);
   
+
+  /**
+   * Is station temporary.
+   */
+  bool is_temporary() {return (temps != TT_TEMPSTATION_NONE);}
+
+  /**
+   * Set temporary status.
+   */
+  void set_temporary(char ts) {temps = ts;}
+
+  /**
+   * Set temporary status.
+   */
+  void set_temporary(const char * name);
+
+  /**
+   * Export station attributes to metapost.
+   */
+
+  void export_mp_flags(FILE * out);
   
+
+  /**
+   * Return 3D splay outline. Process if necessary.
+   */
+
+  thdb3ddata * get_3d_outline();
+
+
 };
 
 
@@ -211,9 +274,7 @@ class thdb1dl {
 typedef std::vector < thdb1dl > thdb1d_leg_vec_type;
 typedef std::list < thdb1d_loop_leg > thdb1d_loop_leg_list_type;
 typedef std::list < thdb1d_loop > thdb1d_loop_list_type;
-
-
-
+typedef std::list < thdb1d_traverse > thdb1d_traverse_list_type;
 
 
 
@@ -233,7 +294,11 @@ class thdb1d {
   
   thdb1d_tree_node  * tree_nodes;
   
-  thdb3ddata d3_data, d3_surface, d3_walls;
+  thdb3ddata d3_data, d3_surface, d3_walls, d3_splay;
+
+  thattr m_station_attr;
+
+  double min_year, max_year;
   
   bool d3_data_parsed;
   
@@ -281,6 +346,8 @@ class thdb1d {
 
   thdb1d_loop_list_type loop_list;
 
+  thdb1d_traverse_list_type traverse_list;
+
   /**
    * Standard constructor.
    */
@@ -357,6 +424,7 @@ class thdb1d {
   void process_xelev();
   
   thdb3ddata * get_3d();
+  thdb3ddata * get_3d_splay();
   thdb3ddata * get_3d_surface();
   thdb3ddata * get_3d_walls();
     
