@@ -32,7 +32,6 @@
 
 #include "thexport.h"
 #include "thsymbolset.h"
-#include "thlayout.h"
 #include "thlang.h"
 
 /**
@@ -44,6 +43,9 @@ enum {
   TT_EXPMAP_OPT_FORMAT,  ///< Output option.
   TT_EXPMAP_OPT_PROJECTION,  ///< Output projection.
   TT_EXPMAP_OPT_LAYOUT,  ///< Layout.
+  TT_EXPMAP_OPT_ENCODING,  ///< Output encoding.
+  TT_EXPMAP_OPT_ENABLE,  ///< Output option.
+  TT_EXPMAP_OPT_DISABLE,  ///< Output option.
 };
 
 
@@ -52,6 +54,10 @@ enum {
  */
  
 static const thstok thtt_expmap_opt[] = {
+  {"-disable", TT_EXPMAP_OPT_DISABLE},
+  {"-enable", TT_EXPMAP_OPT_ENABLE},
+  {"-enc", TT_EXPMAP_OPT_ENCODING},
+  {"-encoding", TT_EXPMAP_OPT_ENCODING},
   {"-fmt", TT_EXPMAP_OPT_FORMAT},
   {"-format", TT_EXPMAP_OPT_FORMAT},
   {"-layout", TT_EXPMAP_OPT_LAYOUT},
@@ -60,6 +66,25 @@ static const thstok thtt_expmap_opt[] = {
   {NULL, TT_EXPMAP_OPT_UNKNOWN}
 };
 
+/**
+ * Model export item options.
+ */
+
+enum {
+  TT_EXPMAP_ITEM_UNKNOWN = 0, 
+  TT_EXPMAP_ITEM_ENTRANCES = 32,
+  TT_EXPMAP_ITEM_ALL = 255,  
+};
+
+/**
+ * Options parsing table.
+ */
+ 
+static const thstok thtt_expmap_items[] = {
+  {"all", TT_EXPMAP_ITEM_ALL},
+  {"entrances", TT_EXPMAP_ITEM_ENTRANCES},
+  {NULL, TT_EXPMAP_ITEM_UNKNOWN}
+};
 
 
 /**
@@ -68,10 +93,16 @@ static const thstok thtt_expmap_opt[] = {
 
 enum {
   TT_EXPMAP_FMT_UNKNOWN = 0,  ///< Unknown option
-  TT_EXPMAP_FMT_SVG,  ///< SVG
-  TT_EXPMAP_FMT_PDF,  ///< PDF
-  TT_EXPMAP_FMT_XVI,  ///< PDF
-  TT_EXPMAP_FMT_3D,  ///< PDF
+  TT_EXPMAP_FMT_SVG,  
+  TT_EXPMAP_FMT_XHTML,
+  TT_EXPMAP_FMT_PDF, 
+  TT_EXPMAP_FMT_XVI,  
+  TT_EXPMAP_FMT_TH2,  
+  TT_EXPMAP_FMT_3D,  
+  TT_EXPMAP_FMT_SHP,  
+  TT_EXPMAP_FMT_KML,  
+  TT_EXPMAP_FMT_DXF,  
+  TT_EXPMAP_FMT_BBOX,  
 };
 
 
@@ -80,9 +111,18 @@ enum {
  */
  
 static const thstok thtt_expmap_fmt[] = {
+  {"bbox", TT_EXPMAP_FMT_BBOX},
+  {"dxf", TT_EXPMAP_FMT_DXF},
+  {"esri", TT_EXPMAP_FMT_SHP},
+  {"kml", TT_EXPMAP_FMT_KML},
   {"pdf", TT_EXPMAP_FMT_PDF},
+  {"shapefile", TT_EXPMAP_FMT_SHP},
+  {"shapefiles", TT_EXPMAP_FMT_SHP},
+  {"shp", TT_EXPMAP_FMT_SHP},
   {"survex", TT_EXPMAP_FMT_3D},
   {"svg", TT_EXPMAP_FMT_SVG},
+  {"th2", TT_EXPMAP_FMT_TH2},
+  {"xhtml", TT_EXPMAP_FMT_XHTML},
   {"xvi", TT_EXPMAP_FMT_XVI},
   {NULL, TT_EXPMAP_FMT_UNKNOWN}
 };
@@ -98,11 +138,19 @@ class thexpmapmpxs {
   public:
   
   FILE * file; ///< output file
+  class thdb2dprj * proj;
   double ms, mx, my, sr, cr, rr; ///< Scale and centering.
-  thsymbolset * symset;
-  thlayout * layout;
+
+  thsymbolset * symset = nullptr;
+  thlayout * layout = nullptr;
+  const char * attr_last_id, * attr_last_survey, * attr_last_scrap;
+  bool attr_last_scrap_centerline;
+
+  double m_shift_x, m_shift_y;
   
-  thexpmapmpxs() : file(NULL), ms(1.0), mx(0.0), my(0.0), sr(0.0), cr(1.0), rr(0.0) {}
+  thexpmapmpxs() : file(NULL), proj(NULL), ms(1.0), mx(0.0), my(0.0), sr(0.0), cr(1.0), rr(0.0),
+    attr_last_id(""), attr_last_survey(""), attr_last_scrap(""), attr_last_scrap_centerline(false),
+    m_shift_x(0.0), m_shift_y(0.0) {}
   
 };
 
@@ -145,22 +193,35 @@ class thexpmap : public thexport {
   
   friend class thexporter;
   
-  int format;  ///< Output format.
-  char * projstr,  ///< Projection string.
+  int format,  ///< Output format.
+    encoding;  ///< Output encoding.
+  const char * projstr,  ///< Projection string.
     * layoutstr;  ///< Layout string.
   class thdb2dprj * projptr;  ///< Projection pointer.
-  class thlayout * layout;  ///< Layout pointer.
   thsymbolset symset;
   
   thbuffer layoutopts;  ///< Layout options buffer.
+
+  unsigned items;  ///< Item types to be exported (As of 29/11/2016 affects only KML entrances)
+  
+  void export_xvi(class thdb2dprj * prj);
+  void export_th2(class thdb2dprj * prj);
   
   void export_pdf(class thdb2dxm * maps, class thdb2dprj * prj);
-	void export_pdf_set_colors(class thdb2dxm * maps, class thdb2dprj * prj);
-	void export_xvi(class thdb2dprj * prj);
-  void export_uni(class thdb2dxm * maps, class thdb2dprj * prj);
   thexpmap_xmps export_mp(thexpmapmpxs * out, class thscrap * scrap, unsigned & startnum, bool outline_mode);  
+
+  void export_pdf_set_colors(class thdb2dxm * maps, class thdb2dprj * prj);
+  void export_pdf_set_colors_new(class thdb2dxm * maps, class thdb2dprj * prj);
+  void export_scrap_backgroud_mesh(class thscrap * cs, class thexpmapmpxs * out, struct scraprecord * r);
+
+  void export_uni(class thdb2dxm * maps, class thdb2dprj * prj);
   void export_uni_scrap(FILE * out, class thscrap * scrap);  
 
+  void export_shp(class thdb2dxm * maps, class thdb2dprj * prj);
+  void export_kml(class thdb2dxm * maps, class thdb2dprj * prj);
+  void export_bbox(class thdb2dxm * maps, class thdb2dprj * prj);
+  void export_dxf(class thdb2dxm * maps, class thdb2dprj * prj);
+
   public:
   
   thexpmap(); ///< Default constructor.
@@ -171,7 +232,7 @@ class thexpmap : public thexport {
    * Parse map export options.
    */
    
-  virtual void parse_options(int & argx, int nargs, char ** args);
+  void parse_options(int & argx, int nargs, char ** args) override;
 
   /**
    * Parse layout options.
@@ -184,21 +245,21 @@ class thexpmap : public thexport {
    * Dump object into file.
    */
    
-  virtual void dump_body(FILE * xf);
+  void dump_body(FILE * xf) override;
 
 
   /**
    * Dump object into file.
    */
    
-  virtual void dump_header(FILE * xf);
+  void dump_header(FILE * xf) override;
   
 
   /**
    * Make export.
    */
    
-  virtual void process_db(class thdatabase * dbp);
+  void process_db(class thdatabase * dbp) override;
 
 
   /**
@@ -216,6 +277,7 @@ class thexpmap : public thexport {
  
 extern double thexpmap_quick_map_export_scale;
 
+void thexpmap_log_log_file(const char * logfpath, const char * on_title, const char * off_title, bool mpbug = false);
 
 #endif
 
