@@ -47,7 +47,7 @@
 #include "thtexfonts.h"
 #include "thlang.h"
 #include "thfilehandle.h"
-#include <libgen.h>
+#include <filesystem>
 #include <thread>
 
 thexpmodel::thexpmodel() {
@@ -216,22 +216,17 @@ void thexpmodel::export_3d_file(class thdatabase * dbp)
   unsigned long i;
   img * pimg;
   img_output_version = 8;
-  thbuffer fnmb;
-  #ifdef THWIN32
-    char title[_MAX_FNAME];
-    _splitpath(this->outpt, NULL, NULL, title, NULL);
-  #elif THLINUX
-    thbuffer bnb;
-    bnb.strcpy(this->outpt);
-    const char * title = basename(bnb.get_buffer());
-  #else
-    const char * title = "cave";
-  #endif
-  fnmb.strcpy(title);  // VG 290316: Set the filename as a cave name instead of "cave". The top-level survey name will be even better
+  std::string fnmb = "cave";
+  // VG 290316: Set the filename as a cave name instead of "cave". The top-level survey name will be even better
+  try {
+    fnmb = std::filesystem::path(this->outpt).filename().string();
+  } catch(const std::exception& e) {
+    thwarning(("unable to obtain output file name -- %s", e.what()))
+  }
   if ((thcfg.outcs >= 0) || (thcfg.outcs < TTCS_UNKNOWN))  // Export the coordinate system data if one is set
-    pimg = img_open_write_cs(fnm, fnmb.get_buffer(), thcs_get_params(thcfg.outcs).c_str(), 1);
+    pimg = img_open_write_cs(fnm, fnmb.c_str(), thcs_get_params(thcfg.outcs).c_str(), 1);
   else
-    pimg = img_open_write(fnm, fnmb.get_buffer(), 1);
+    pimg = img_open_write(fnm, fnmb.c_str(), 1);
      
   if (!pimg) {
     thwarning(("can't open %s for output",fnm))
