@@ -33,7 +33,6 @@
 #include <unistd.h>
 #endif
 
-#include <sys/stat.h>
 #include "thbuffer.h"
 #include "thmbuffer.h"
 #include "thparse.h"
@@ -85,11 +84,6 @@ class thinput {
     int encoding;  /// Current file encoding.
     ifile * prev_ptr;  /// Pointer to the upper file.
     std::unique_ptr<ifile> next_ptr;  /// Pointer to the lower file.
-#ifdef THMSVC
-    struct _stat st;const char * thinput::get_cif_abspath()
-#else
-    struct stat st{};
-#endif
       
     
     /**
@@ -109,13 +103,7 @@ class thinput {
      * Check if file statistics are equal.
      */
      
-    bool is_equal(
-#ifdef THMSVC
-struct _stat
-#else
-struct stat
-#endif
-         * buf);
+    bool is_equal(ifile* f);
     
   };
   
@@ -301,7 +289,7 @@ struct stat
    * Return current absolute input file path.
    */  
    
-  const char * get_cif_abspath(const char * fname = "");
+  std::string get_cif_abspath(const char * fname = "");
   
   /**
    * Return current input line number.
