@@ -1,7 +1,7 @@
 #!/usr/bin/perl
-## usage: 
-## ./process.pl - generate .cxx,.h sources from texts.txt
-## ./process.pl update - find new strings to translate and load all 
+## usage:
+## ./process.pl - generate .h sources from texts.txt
+## ./process.pl update - find new strings to translate and load all
 ##   texts_xy.txt into texts.txt
 ## ./process.pl export[-empty] xy - export texts_xy.txt with texts
 ##   for translation. If export-empty is used, only untranslated strings
@@ -87,13 +87,13 @@ sub write_lang_file {
   my %hr = %{$href};
 
   open(OUT,">$fn") || die("error: can't open $fn for output\n");
-  
+
   my %sh = ();
   for $key (keys %hr) {
 #    print "SORTING: $key -> $hr{$key}{therion}\n";
     $sh{$hr{$key}{therion}} = $key;
   }
-  
+
   @nkeys = sort {$a <=> $b} (keys %sh);
   for $key (@nkeys) {
     print OUT $hr{$sh{$key}}{"therion-cmnt"};
@@ -106,15 +106,15 @@ sub write_lang_file {
     }
     print OUT "\n";
   }
-  
+
   close(OUT);
-  
+
 }
 
 sub write_sources {
   my $href = shift;
   my %hr = %{$href};
-  
+
   # vytvorime si zoznam jazykov a ich alternativ
   my %lngs = ();
   my $key;
@@ -130,25 +130,25 @@ sub write_sources {
       }
     }
   }
-  
+
   my @langs = (sort keys %lngs);
   my $i;
   my $lcode;
-  $languages = "enum {\n  THLANG_UNKNOWN = -1,\n";
-  $langcxxid = "static const thlang_pchar thlang__cxxids []  = {\n";
+  $languages = "enum {\n  THLANG_SYSTEM = -2,\n  THLANG_UNKNOWN = -1,\n";
+  $langcxxid = "static const thlang_pchar thlang_cxxids []  = {\n";
   $langparse = "static const thstok thtt_lang [] = {\n";
   for ($i = 0; $i <= $#langs; $i++) {
     $lcode = "THLANG_" . uc($langs[$i]);
     $langcxxid .= "  \"$lcode\",\n";
-    $langparse .= "  {\"$langs[$i]\", $lcode},\n"; 
+    $langparse .= "  {\"$langs[$i]\", $lcode},\n";
     $languages .= "  $lcode = $i,\n";
     $lngs{$langs[$i]} = $lcode;
   }
   $languages .= "};\n";
   $langcxxid .= "};\n";
   $langparse .= "  {NULL, THLANG_UNKNOWN},\n};\n";
-  
-  my $alternatives = "static const int thlang__alternatives [] = {\n";
+
+  my $alternatives = "static const int thlang_alternatives [] = {\n";
   for ($i = 0; $i <= $#langs; $i++) {
     $lkey = $langs[$i];
     if ((length($lkey) > 2) && (defined($lngs{substr($lkey,0,2)}))) {
@@ -158,11 +158,11 @@ sub write_sources {
     }
   }
   $alternatives .= "};\n";
-  
+
   @texts = sort keys %hr;
 
-  $textparse = "static const thstok thtt__texts [" . ($#texts + 2) . "] = {\n";
-  $texttable = "static const thlang_pchar thlang__translations [" . ($#texts + 1) . "][" . ($#langs + 1) . "] = {\n";
+  $textparse = "static const thstok thtt_texts [" . ($#texts + 2) . "] = {\n";
+  $texttable = "static thlang_pchar thlang_translations [" . ($#texts + 1) . "][" . ($#langs + 1) . "] = {\n";
   $i = 0;
   my $nlkey;
   foreach $key (@texts) {
@@ -189,8 +189,8 @@ sub write_sources {
   }
   $textparse .= "  {NULL, -1},\n};\n";
   $texttable .= "};\n";
-  
-  # exportujeme h subor
+
+  # exportujeme thlangdata.h
   open(OUT,">../thlangdata.h") || die("error: can't open thlangdata.h for output\n");
   print OUT <<ENDOUT;
 /**
@@ -199,26 +199,29 @@ sub write_sources {
  *
  * THIS FILE IS GENERATED AUTOMATICALLY, DO NOT MODIFY IT !!!
  */
-  
- 
+#pragma once
+
 $languages
- 
- 
+
+
 ENDOUT
   close(OUT);
-  
-  # exportujeme cxx subor
-  open(OUT,">../thlangdata.cxx") || die("error: can't open thlangdata.cxx for output\n");
+
+  # exportujeme thlangdatafields.h
+  open(OUT,">../thlangdatafields.h") || die("error: can't open thlangdatafields.h for output\n");
   print OUT <<ENDOUT;
 /**
- * \@file thlangdata.cxx
+ * \@file thlangdatafields.h
  * Therion language translations module.
  *
  * THIS FILE IS GENERATED AUTOMATICALLY, DO NOT MODIFY IT !!!
  */
 
- 
-typedef char * thlang_pchar;
+#include "thlangdata.h"
+#include "thparse.h"
+#include <cstddef>
+
+typedef const char * thlang_pchar;
 
 
 $langparse
@@ -234,19 +237,19 @@ $textparse
 
 
 $texttable
- 
- 
+
+
 ENDOUT
-  
+
   close(OUT);
-  
+
 }
 
 
 sub update_todo_list {
   my $href = shift;
   my %hr = %{$href};
-  
+
   # prejde vsetky kluce a priradi im vysoke cisla
   my $maxnm = -1;
   foreach $key (keys %hr) {
@@ -255,11 +258,11 @@ sub update_todo_list {
     }
   }
   $maxnm = -1 - $maxnm;
-  
+
   foreach $key (keys %hr) {
     $hr{$key}{therion} += $maxnm;
   }
-  
+
   my $i = 0;
   foreach $fn (@trans_sources) {
     print "updating definitions from $fn ...";
@@ -271,27 +274,27 @@ sub update_todo_list {
     @lines = split(/\n/,$ln);
     foreach $ln (@lines) {
       $i++;
-      if ($ln =~ /^thT\(\"(.*)\"/) {
+      if ($ln =~ /^thT\(\"([^"]+)\"/) {
         $hr{$1}{therion} = $i;
       }
     }
     print " done\n";
   }
-  
+
   #vypise warningy
   foreach $key (keys %hr) {
     if ($hr{$key}{therion} < 0) {
       warn("warning: expression \"$key\" doesn't need to be translated\n");
     }
   }
-  
+
   return \%hr;
 }
 
 
 sub backup_file {
   my $fn = shift;
-  
+
   open(INP,"$fn") || die("error: can't open $fn for input\n");
   my @lines = <INP>;
   close(INP);
@@ -308,11 +311,11 @@ sub export_language {
   open(OUT,">$fn") || die("error: can't open $fn for output\n");
 
   my %sh = ();
-  
+
   for $key (keys %hr) {
     $sh{$hr{$key}{therion}} = $key;
   }
-  
+
   my @nkeys = sort {$a <=> $b} (keys %sh);
   for $key (@nkeys) {
     my $toexp = 1;
@@ -340,7 +343,7 @@ sub export_language {
   }
 
   close(OUT);
-  
+
 }
 
 
