@@ -34,6 +34,7 @@
 #include "thlayoutln.h"
 #include "thsymbolset.h"
 #include "thlocale.h"
+#include "thlayoutclr.h"
 #include <list>
 
 /**
@@ -89,6 +90,17 @@ enum {
   TT_LAYOUT_COLOR_LEGEND = 2045,
   TT_LAYOUT_MAP_HEADER_BG = 2046,
 	TT_LAYOUT_UNITS = 2047,
+	TT_LAYOUT_SURVEY_LEVEL = 2048,
+	TT_LAYOUT_SKETCHES = 2049,
+	TT_LAYOUT_NORTH = 2050,
+  TT_LAYOUT_MAP_IMAGE = 2051,
+  TT_LAYOUT_GRID_COORDS = 2052,
+  TT_LAYOUT_SYMBOL_COLOR = 2053,
+  TT_LAYOUT_FONT_SETUP = 2054,
+  TT_LAYOUT_MIN_SYMBOL_SCALE = 2055,
+  TT_LAYOUT_COLOR_MODEL = 2056,
+  TT_LAYOUT_COLOR_PROFILE = 2057,
+  TT_LAYOUT_SMOOTH_SHADING = 2058,
 };
 
 
@@ -116,6 +128,30 @@ static const thstok thtt_layout_grid[] = {
 };
 
 
+/**
+ * Layout grid tokens.
+ */
+
+enum {
+  TT_LAYOUT_GRIDCOORDS_UNKNOWN = 0,
+  TT_LAYOUT_GRIDCOORDS_OFF,
+  TT_LAYOUT_GRIDCOORDS_BORDER,
+  TT_LAYOUT_GRIDCOORDS_ALL,
+};
+
+
+/**
+ * Layout grid token table.
+ */
+ 
+static const thstok thtt_layout_gridcoords[] = {
+  {"all", TT_LAYOUT_GRIDCOORDS_ALL},
+  {"border", TT_LAYOUT_GRIDCOORDS_BORDER},
+  {"off", TT_LAYOUT_GRIDCOORDS_OFF},
+  {NULL, TT_LAYOUT_GRIDCOORDS_UNKNOWN}
+};
+
+
 /**
  * Layout map-item token table.
  */
@@ -150,6 +186,32 @@ static const thstok thtt_layout_legend[] = {
 };
 
 
+/**
+ * Layout color legend tokens.
+ */
+
+enum {
+  TT_LAYOUT_COLORLEGEND_UNKNOWN = 0,
+  TT_LAYOUT_COLORLEGEND_OFF,
+  TT_LAYOUT_COLORLEGEND_DISCRETE,
+  TT_LAYOUT_COLORLEGEND_SMOOTH,
+};
+
+
+/**
+ * Layout color legend token table.
+ */
+
+static const thstok thtt_layout_colorlegend[] = {
+  {"discrete", TT_LAYOUT_COLORLEGEND_DISCRETE},
+  {"off", TT_LAYOUT_COLORLEGEND_OFF},
+  {"on", TT_LAYOUT_COLORLEGEND_SMOOTH}, // legacy option
+  {"smooth", TT_LAYOUT_COLORLEGEND_SMOOTH},
+  {NULL, TT_LAYOUT_LEGEND_UNKNOWN},
+};
+
+
+
 /**
  * Layout debug tokens.
  */
@@ -159,6 +221,7 @@ enum {
   TT_LAYOUT_DEBUG_STATIONS,
   TT_LAYOUT_DEBUG_JOINS,
   TT_LAYOUT_DEBUG_SCRAPNAMES,
+  TT_LAYOUT_DEBUG_STATIONNAMES,
   TT_LAYOUT_DEBUG_ALL,
   TT_LAYOUT_DEBUG_UNKNOWN,
 };
@@ -175,6 +238,7 @@ static const thstok thtt_layout_debug[] = {
   {"on", TT_LAYOUT_DEBUG_ALL},
   {"scrap-names", TT_LAYOUT_DEBUG_SCRAPNAMES},
   {"second", TT_LAYOUT_DEBUG_JOINS},
+  {"station-names", TT_LAYOUT_DEBUG_STATIONNAMES},
   {NULL, TT_LAYOUT_DEBUG_UNKNOWN},
 };
 
@@ -203,6 +267,27 @@ static const thstok thtt_layout_surface[] = {
 };
 
 
+/**
+ * Layout smooth shading tokens.
+ */
+
+enum {
+  TT_LAYOUT_SMOOTHSHADING_OFF,
+  TT_LAYOUT_SMOOTHSHADING_QUICK,
+  TT_LAYOUT_SMOOTHSHADING_UNKNOWN,
+};
+
+
+/**
+ * Layout smooth shading token table.
+ */
+ 
+static const thstok thtt_layout_smoothshading[] = {
+  {"off", TT_LAYOUT_SMOOTHSHADING_OFF},
+  {"quick", TT_LAYOUT_SMOOTHSHADING_QUICK},
+  {NULL, TT_LAYOUT_SMOOTHSHADING_UNKNOWN},
+};
+
 
 
 
@@ -262,6 +347,7 @@ enum {
   TT_LAYOUT_CODE_SYMBOL_ASSIGN,
   TT_LAYOUT_CODE_SYMBOL_HIDE,
   TT_LAYOUT_CODE_SYMBOL_SHOW,
+  TT_LAYOUT_CODE_SYMBOL_COLOR,
   TT_LAYOUT_CODE_MAP_ITEM,
 };
 
@@ -279,6 +365,51 @@ static const thstok thtt_layout_code[] = {
 };
 
 
+/**
+ * Layout north tokens.
+ */
+
+enum {
+  TT_LAYOUT_NORTH_UNKNOWN = 0,
+  TT_LAYOUT_NORTH_TRUE,
+  TT_LAYOUT_NORTH_GRID,
+};
+
+
+/**
+ * Layout color token table.
+ */
+ 
+static const thstok thtt_layout_north[] = {
+  {"grid", TT_LAYOUT_NORTH_GRID},
+  {"true", TT_LAYOUT_NORTH_TRUE},
+  {NULL, TT_LAYOUT_NORTH_UNKNOWN}
+};
+
+/**
+ * Layout length statistics tokens.
+ */
+
+enum {
+  TT_LAYOUT_LENSTAT_UNKNOWN = 0,
+  TT_LAYOUT_LENSTAT_OFF,
+  TT_LAYOUT_LENSTAT_HIDE,
+  TT_LAYOUT_LENSTAT_ON,
+};
+
+
+/**
+ * Layout color token table.
+ */
+ 
+static const thstok thtt_layout_lenstat[] = {
+  {"hide", TT_LAYOUT_LENSTAT_HIDE},
+  {"off", TT_LAYOUT_LENSTAT_OFF},
+  {"on", TT_LAYOUT_LENSTAT_ON},
+  {NULL, TT_LAYOUT_LENSTAT_UNKNOWN}
+};
+
+
 /**
  * Layout color tokens.
  */
@@ -289,6 +420,7 @@ enum {
   TT_LAYOUT_COLOR_MAP_FG,
   TT_LAYOUT_COLOR_PREVIEW_BELOW,
   TT_LAYOUT_COLOR_PREVIEW_ABOVE,
+  TT_LAYOUT_COLOR_LABELS,
 };
 
 
@@ -297,6 +429,7 @@ enum {
  */
  
 static const thstok thtt_layout_color[] = {
+  {"labels", TT_LAYOUT_COLOR_LABELS},
   {"map-bg", TT_LAYOUT_COLOR_MAP_BG},
   {"map-fg", TT_LAYOUT_COLOR_MAP_FG},
   {"preview-above", TT_LAYOUT_COLOR_PREVIEW_ABOVE},
@@ -312,47 +445,34 @@ static const thstok thtt_layout_color[] = {
 enum {
   TT_LAYOUT_CCRIT_UNKNOWN = 0,
   TT_LAYOUT_CCRIT_ALTITUDE,
+  TT_LAYOUT_CCRIT_DEPTH,
   TT_LAYOUT_CCRIT_TOPODATE,
   TT_LAYOUT_CCRIT_EXPLODATE,
   TT_LAYOUT_CCRIT_MAP,
+  TT_LAYOUT_CCRIT_SCRAP,
+  TT_LAYOUT_CCRIT_SURVEY,
 };
 
 static const thstok thtt_layout_ccrit[] = {
   {"altitude", TT_LAYOUT_CCRIT_ALTITUDE},
-//  {"topo-date", TT_LAYOUT_CCRIT_TOPODATE},
-//  {"explo-date", TT_LAYOUT_CCRIT_EXPLODATE},
+  {"altitudes", TT_LAYOUT_CCRIT_ALTITUDE},
+  {"depth", TT_LAYOUT_CCRIT_DEPTH},
+  {"explo-date", TT_LAYOUT_CCRIT_EXPLODATE},
   {"map", TT_LAYOUT_CCRIT_MAP},
+  {"maps", TT_LAYOUT_CCRIT_MAP},
+  {"scrap", TT_LAYOUT_CCRIT_SCRAP},
+  {"scraps", TT_LAYOUT_CCRIT_SCRAP},
+  {"topo-date", TT_LAYOUT_CCRIT_TOPODATE},
   {NULL, TT_LAYOUT_CCRIT_UNKNOWN}
 };
 
 
-/**
- * Layout color tables.
- */
-
-enum {
-  TT_LAYOUT_CTABLE_UNKNOWN = 0,
-  TT_LAYOUT_CTABLE_HSV,
-};
-
-static const thstok thtt_layout_ctable[] = {
-  {"hsv", TT_LAYOUT_CTABLE_HSV},
-  {NULL, TT_LAYOUT_CTABLE_UNKNOWN}
-};
-
-enum {
-  TT_LAYOUT_CMODE_AUTO,
-  TT_LAYOUT_CMODE_TABLE,
-  TT_LAYOUT_CMODE_MANUAL,
-};
-
-
 
 class thlayout_copy_src {
 
   public:
   
-  char * srcn;
+  const char * srcn;
   thlayout * srcptr;
   class thlayout_copy_src * next_src;
   thlayout_copy_src () : srcn(NULL), srcptr(NULL), next_src(NULL) {};
@@ -369,8 +489,12 @@ static const thstok thtt_layout_opt[] = {
   {"code",TT_LAYOUT_CODE},
   {"color",TT_LAYOUT_COLOR},
   {"color-legend",TT_LAYOUT_COLOR_LEGEND},
+  {"color-model", TT_LAYOUT_COLOR_MODEL},
+  {"color-profile", TT_LAYOUT_COLOR_PROFILE},
   {"colour",TT_LAYOUT_COLOR},
   {"colour-legend",TT_LAYOUT_COLOR_LEGEND},
+  {"colour-model", TT_LAYOUT_COLOR_MODEL},
+  {"colour-profile", TT_LAYOUT_COLOR_PROFILE},
   {"copy",TT_LAYOUT_COPY},
   {"debug",TT_LAYOUT_DEBUG},
   {"doc-author",TT_LAYOUT_DOC_AUTHOR},
@@ -379,7 +503,9 @@ static const thstok thtt_layout_opt[] = {
   {"doc-title",TT_LAYOUT_DOC_TITLE},
   {"endcode",TT_LAYOUT_ENDCODE},
   {"exclude-pages",TT_LAYOUT_EXCLUDE_PAGES},
+  {"fonts-setup", TT_LAYOUT_FONT_SETUP},
   {"grid",TT_LAYOUT_GRID},
+  {"grid-coords",TT_LAYOUT_GRID_COORDS},
   {"grid-origin",TT_LAYOUT_GRID_ORIGIN},
   {"grid-size",TT_LAYOUT_GRID_SIZE},
   {"language",TT_LAYOUT_LANG},
@@ -390,8 +516,11 @@ static const thstok thtt_layout_opt[] = {
   {"map-comment",TT_LAYOUT_DOC_COMMENT},
   {"map-header",TT_LAYOUT_MAP_HEADER},
   {"map-header-bg",TT_LAYOUT_MAP_HEADER_BG},
+  {"map-image",TT_LAYOUT_MAP_IMAGE},
+  {"min-symbol-scale", TT_LAYOUT_MIN_SYMBOL_SCALE},
   {"nav-factor",TT_LAYOUT_NAV_FACTOR},
   {"nav-size",TT_LAYOUT_NAV_SIZE},
+  {"north",TT_LAYOUT_NORTH},
   {"opacity",TT_LAYOUT_OPACITY},
   {"origin",TT_LAYOUT_ORIGIN},
   {"origin-label",TT_LAYOUT_ORIGIN_LABEL},
@@ -404,10 +533,15 @@ static const thstok thtt_layout_opt[] = {
   {"scale", TT_LAYOUT_SCALE},
   {"scale-bar", TT_LAYOUT_SCALE_BAR},
   {"size", TT_LAYOUT_SIZE},
+  {"sketches", TT_LAYOUT_SKETCHES},
+  {"smooth-shading",TT_LAYOUT_SMOOTH_SHADING},
   {"statistics",TT_LAYOUT_MAP_ITEM},
   {"surface",TT_LAYOUT_SURFACE},
   {"surface-opacity",TT_LAYOUT_SURFACE_OPACITY},
+  {"survey-level",TT_LAYOUT_SURVEY_LEVEL},
   {"symbol-assign", TT_LAYOUT_SYMBOL_ASSIGN},
+  {"symbol-color", TT_LAYOUT_SYMBOL_COLOR},
+  {"symbol-colour", TT_LAYOUT_SYMBOL_COLOR},
   {"symbol-hide", TT_LAYOUT_SYMBOL_HIDE},
   {"symbol-set", TT_LAYOUT_SYMBOL_DEFAULTS},
   {"symbol-show", TT_LAYOUT_SYMBOL_SHOW},
@@ -418,30 +552,17 @@ static const thstok thtt_layout_opt[] = {
 };
 
 
-/**
- * layout color class.
- */
 
-struct thlayout_color {
-  double R, G, B;
-  bool defined;
-  void parse(char * str);
-  thlayout_color() : R(1.0), G(1.0), B(1.0), defined(false) {}
+struct thlayout_map_image {
+  double m_x, m_y;
+  const char * m_fn;
+  int m_align;
+  thlayout_map_image() : m_x(0.0), m_y(0.0), m_fn(""), m_align(TT_LAYOUT_MAP_HEADER_OFF) {}
+  bool defined();
+  void parse(char ** args, const char * cpath);
 };
 
-
-/**
- * layout color value class
- */
-
-struct thlayout_color_value {
-  thlayout_color color;
-  double vdbl;
-  thdate vdate;
-  char * vname;
-};
-
-typedef std::list<thlayout_color_value> thlayout_ct;
+typedef std::list<thlayout_map_image> thlayout_map_image_list;
 
 
 /**
@@ -451,45 +572,60 @@ typedef std::list<thlayout_color_value> thlayout_ct;
 class thlayout : public thdataobject {
 
   public:
+
+  class thconfig * m_pconfig;
+
+  class thlookup * m_lookup;
     
   double scale, scale_bar, base_scale, ox, oy, oz, hsize, vsize, paphs, papvs, paghs, pagvs, marls, marts, gxs, gys, gzs, gox, goy, goz, navf, overlap, opacity,
     map_header_x, map_header_y, legend_width, surface_opacity, rotate;
   
-  char * olx, * oly, 
+  int o_cs, go_cs;
+
+  const char * olx, * oly, 
     * doc_title, * doc_comment, * doc_author, * doc_subject, * doc_keywords, * excl_list;
   
   unsigned navsx, navsy, ownp, legend_columns;
   
   char grid, ccode;
   
-  int legend, color_legend, map_header, lang, max_explos, max_topos, max_cartos, max_copys,
-    debug, surface;
+  int legend, color_legend, map_header, lang, north, max_explos, max_topos, max_cartos, max_copys,
+    debug, survey_level, surface, grid_coords, color_model, smooth_shading;
+
+  const char * color_profile_rgb, * color_profile_cmyk, * color_profile_gray;
   
   thlayout_color color_map_bg, color_map_fg, color_preview_below, color_preview_above;
-  int color_crit, // none, altitude, ...
-   color_mode, // auto - values and colors, table - colors, manual
-   color_table; // hsv, cool, hot ...
-  thlayout_ct color_values; // color table
+  int color_crit; // none, altitude, ...
+  const char * color_crit_fname;
+
+  double min_symbol_scale, font_setup[5];
+
   
   thlayoutln * first_line, * last_line;
   
   bool titlep, transparency, layers, pgsnum, lock, excl_pages, page_grid, 
-    explo_lens, topo_lens, map_header_bg;
+    map_header_bg, sketches, color_labels;
+
+  int explo_lens, topo_lens, carto_lens, copy_lens;
 		
 	thlocale units;
+
+  thlayout_map_image_list map_image_list;
   
-  bool def_grid_size, def_grid_origin, def_nav_factor, def_nav_size, 
+  int def_grid_size, def_grid_origin, def_nav_factor, def_nav_size, 
     def_opacity, def_transparency, def_layers, def_base_scale,
-    def_rotate,
+    def_rotate, def_sketches, def_north,
     def_origin, def_origin_label, def_overlap, def_own_pages,
     def_page_numbers, def_page_setup, def_scale, def_size, def_title_pages,
     def_tex_lines, def_doc_title, def_doc_comment, def_doc_author, def_doc_subject,
     def_doc_keywords, def_excl_pages, def_grid, def_page_grid,
     def_legend, def_color_legend, def_legend_width, def_legend_columns, 
     def_map_header, def_lang, def_scale_bar, def_map_header_bg,
-    redef_base_scale, def_max_explos, def_max_topos, def_max_cartos,
-    def_max_copys, def_explo_lens, def_topo_lens, def_debug, def_surface,
-    def_surface_opacity, def_units;
+    def_max_explos, def_max_topos, def_max_cartos,
+    def_max_copys, def_explo_lens, def_topo_lens, def_debug, def_survey_level, def_surface,
+    def_surface_opacity, def_units, def_grid_coords, def_color_labels,
+    def_font_setup, def_min_symbol_scale, def_color_model, def_carto_lens, def_copy_lens,
+	def_color_profile_rgb, def_color_profile_cmyk, def_color_profile_gray, def_smooth_shading;
     
   
   thlayout_copy_src * first_copy_src, * last_copy_src;
@@ -512,76 +648,67 @@ class thlayout : public thdataobject {
    * Return class identifier.
    */
   
-  virtual int get_class_id();
+  int get_class_id() override;
   
   
   /**
    * Return class name.
    */
    
-  virtual char * get_class_name() {return "thlayout";};
+  const char * get_class_name() override {return "thlayout";};
   
   
   /**
    * Return true, if son of given class.
    */
   
-  virtual bool is(int class_id);
+  bool is(int class_id) override;
   
   
   /**
    * Return number of command arguments.
    */
    
-  virtual int get_cmd_nargs();
+  int get_cmd_nargs() override;
   
   
   /**
    * Return command name.
    */
    
-  virtual char * get_cmd_name();
+  const char * get_cmd_name() override;
   
   
   /**
    * Return command end option.
    */
    
-  virtual char * get_cmd_end();
+  const char * get_cmd_end() override;
   
   
   /**
    * Return option description.
    */
    
-  virtual thcmd_option_desc get_cmd_option_desc(char * opts);
+  thcmd_option_desc get_cmd_option_desc(const char * opts) override;
   thcmd_option_desc get_default_cod(int id);
   
   /**
    * Set command option.
    *
    * @param cod Command option description.
-   * @param args Option arguments arry.
+   * @param args Option arguments array.
    * @param argenc Arguments encoding.
    */
    
-  virtual void set(thcmd_option_desc cod, char ** args, int argenc, unsigned long indataline);
-
-
-  /**
-   * Delete this object.
-   *
-   * @warn Always use this methos instead of delete function.
-   */
-   
-  virtual void self_delete();
+  void set(thcmd_option_desc cod, char ** args, int argenc, unsigned long indataline) override;
 
 
   /**
    * Print object properties.
    */
    
-  virtual void self_print_properties(FILE * outf); 
+  void self_print_properties(FILE * outf) override; 
   
 
   /**
@@ -595,7 +722,7 @@ class thlayout : public thdataobject {
    * Get context for object.
    */
    
-  virtual int get_context();
+  int get_context() override;
   
   /**
    * Parse option with 3 or 2 or 1 numbers and units.
@@ -611,13 +738,20 @@ class thlayout : public thdataobject {
    
   void export_config(FILE * o, thdb2dprj * prj, double x_scale, double x_origin_shx, double x_origin_shy);
   
+
+  /**
+   * Export font size.
+   */
+  void export_mptex_font_size(FILE * o, class th2ddataobject * obj, bool print_default_scale);
+
+
   /**
    * Export main tex file.
    */
    
   void export_pdftex(FILE * o, thdb2dprj * prj, char mode);
   void export_mpost(FILE * o);
-  void export_mpost_symbols(FILE * o, class thsymbolset * symset);
+  void export_mpost_symbols(FILE * o, struct thsymbolset * symset);
   
   /**
    * Process copy.
@@ -628,12 +762,20 @@ class thlayout : public thdataobject {
   bool is_debug_stations();
   bool is_debug_joins();
   bool is_debug_scrapnames();
+  bool is_debug_stationnames();
+
   
   /**
    * set LAYOUT variable.
    */
    
   void set_thpdf_layout(thdb2dprj * prj, double x_scale, double x_origin_shx, double x_origin_shy);
+
+  /**
+   * Convert all points in object.
+   */
+
+  void convert_all_cs() override;
    
 
 };
