@@ -32,12 +32,16 @@
 #include "thscrap.h"
 #include "thobjectname.h"
 #include "thdb2dmi.h"
+#include "thtflength.h"
+#include "thconfig.h"
+
 
 thmap::thmap()
 {
   this->first_item = NULL;
   this->last_item = NULL;
   
+  this->expl_projection = NULL;
   this->projection_id = 0;
   this->last_level = 1;
   this->is_basic = true;
@@ -45,7 +49,7 @@ thmap::thmap()
   this->selection_mode = TT_MAPITEM_UNKNOWN;
   this->selection_xs = NULL;
   
-  this->z = 0.0;
+  this->z = thnan;
   this->nz = -1;
 }
 
@@ -75,19 +79,19 @@ int thmap::get_cmd_nargs()
 }
 
 
-char * thmap::get_cmd_end()
+const char * thmap::get_cmd_end()
 {
   return "endmap";
 }
 
 
-char * thmap::get_cmd_name()
+const char * thmap::get_cmd_name()
 {
   return "map";
 }
 
 
-thcmd_option_desc thmap::get_cmd_option_desc(char * opts)
+thcmd_option_desc thmap::get_cmd_option_desc(const char * opts)
 {
   int id = thmatch_token(opts, thtt_map_opt);
   if (id == TT_MAP_UNKNOWN)
@@ -107,6 +111,7 @@ thcmd_option_desc thmap::get_cmd_option_desc(char * opts)
 
 void thmap::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long indataline)
 {
+  thdb2dprjpr projection;
 
   if (cod.id == 1)
     cod.id = TT_DATAOBJECT_NAME;
@@ -119,11 +124,25 @@ void thmap::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long i
         this->db->db2d.mbf.get_buffer());
       break;
     
+    case TT_MAP_PROJECTION:
+      projection = this->db->db2d.parse_projection(*args);
+      if (!projection.parok)
+        ththrow("invalid parameters of projection");
+      this->expl_projection = projection.prj;
+      break;
     
     case TT_MAP_BREAK:
       this->last_level++;
       break;
 
+//    case TT_MAP_COLOUR:
+//      this->colour.parse(args[0]);
+//      break;
+
+	case TT_MAP_SURVEY:
+	  thparse_objectname(this->asoc_survey, &(this->db->buff_stations), args[0], this);
+	  break;
+
     case TT_MAP_PREVIEW:
       this->parse_preview(args);
       break;
@@ -136,11 +155,6 @@ void thmap::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long i
 }
 
 
-void thmap::self_delete()
-{
-  delete this;
-}
-
 void thmap::self_print_properties(FILE * outf)
 {
   thdataobject::self_print_properties(outf);
@@ -159,11 +173,12 @@ void thmap::self_print_properties(FILE * outf)
   }
 }
 
+thmbuffer mapitmmb;
 
 void thmap::parse_item(int npar, char ** pars)
 {
-  if (npar != 1)
-    ththrow(("one map or scrap name per line allowed"))
+  if ((npar != 1) && (npar != 3))
+    ththrow("invalid map item");
   thdb2dmi * citem = this->db->db2d.insert_map_item();
   citem->itm_level = this->last_level;
   citem->source = this->db->csrc;
@@ -178,6 +193,31 @@ void thmap::parse_item(int npar, char ** pars)
     citem->prev_item = this->last_item;
     this->last_item = citem;
   }
+  if ((npar == 3) && (thcfg.use_maps_offset)) {
+    thsplit_words(&mapitmmb, pars[1]);
+    char ** ss;
+    size_t nw;
+    int sv;
+    nw = mapitmmb.get_size();
+    ss = mapitmmb.get_buffer();
+    if ((nw < 2) || (nw > 3))
+      ththrow("invalid map shift specification -- {}", pars[1]);
+
+    thtflength l;
+    thparse_double(sv, citem->m_shift.m_x, ss[0]);
+    if (sv != TT_SV_NUMBER)
+      ththrow("not a number -- {}", ss[0]);
+    thparse_double(sv, citem->m_shift.m_y, ss[1]);
+    if (sv != TT_SV_NUMBER)
+      ththrow("not a number -- {}", ss[1]);
+    if (nw > 2)
+      l.parse_units(ss[2]);
+    citem->m_shift.m_x = l.transform(citem->m_shift.m_x);
+    citem->m_shift.m_y = l.transform(citem->m_shift.m_y);
+    citem->m_shift.m_preview = thmatch_token(pars[2],thtt_2dmi);
+    if ((citem->m_shift.m_preview == TT_MAPITEM_UNKNOWN) || (citem->m_shift.m_preview == TT_MAPITEM_NORMAL))
+      ththrow("unsupported preview type -- {}", pars[2]);
+  }
 }
 
 
@@ -187,8 +227,13 @@ void thmap::parse_preview(char ** pars)
   citem->source = this->db->csrc;
   citem->psurvey = this->db->get_current_survey();
   citem->type = thmatch_token(*pars,thtt_2dmi);
-  if (citem->type == TT_MAPITEM_UNKNOWN)
-    ththrow(("invalid preview type -- %s", *pars))
+  switch (citem->type) {
+    case TT_MAPITEM_ABOVE:
+    case TT_MAPITEM_BELOW:
+      break;
+    default:
+      ththrow("unsupported preview type -- {}", *pars);
+  }
   thparse_objectname(citem->name,& this->db->buff_stations, pars[1]);
   if (this->last_item == NULL) {
     this->first_item = citem;
@@ -213,12 +258,16 @@ void thmap::calc_z() {
       switch (mi->object->get_class_id()) {
         case TT_MAP_CMD:
           ((thmap*)mi->object)->calc_z();
-          this->z += ((thmap*)mi->object)->z;
-          this->nz++;
+          if (((thmap*)mi->object)->nz > 0) {
+            this->z += ((thmap*)mi->object)->z;
+            this->nz++;
+          }
           break;
         case TT_SCRAP_CMD:
-          this->z += ((thscrap*)mi->object)->z;
-          this->nz++;
+          if (!thisnan(((thscrap*)mi->object)->z)) {
+            this->z += ((thscrap*)mi->object)->z;
+            this->nz++;
+          }
           break;
       }
     }
@@ -226,6 +275,8 @@ void thmap::calc_z() {
   }
   if (this->nz > 0) {
     this->z /= double(this->nz);
+  } else {
+    this->z = thnan;
   }
 }
 
