@@ -57,11 +57,14 @@ enum {
   TT_SELECTOR_RECURSIVE,
   TT_SELECTOR_MAP_LEVEL,
   TT_SELECTOR_CHAPTER_LEVEL,
+  TT_SELECTOR_COLOR,
   TT_SELECTOR_UNKNOWN,
 };
 
 static const thstok thtt_selector_opts[] = {
   {"-chapter-level", TT_SELECTOR_CHAPTER_LEVEL},
+  {"-color", TT_SELECTOR_COLOR},
+  {"-colour", TT_SELECTOR_COLOR},
   {"-map-level", TT_SELECTOR_MAP_LEVEL},
   {"-recursive", TT_SELECTOR_RECURSIVE},
   {NULL, TT_SELECTOR_UNKNOWN}
@@ -76,11 +79,11 @@ void thselector::parse_selection (bool usid, int nargs, char ** args)
   thselector_item itm;
   itm.unselect = usid;
   if (nargs < 1)
-    ththrow(("not enough command arguments"))
+    ththrow("not enough command arguments");
 
   // set object name
   if (strlen(*args) == 0)
-    ththrow(("empty object name not allowed"))
+    ththrow("empty object name not allowed");
   itm.name = this->cfgptr->get_str_buff()->append(*args);
   itm.src_name = this->cfgptr->get_db()->strstore(this->cfgptr->get_cfg_file()->get_cif_name(), true);
   itm.src_ln = this->cfgptr->get_cfg_file()->get_cif_line_number();
@@ -90,6 +93,7 @@ void thselector::parse_selection (bool usid, int nargs, char ** args)
   int aid;
   for(aid = 1; aid < nargs; aid++) {
     switch (thmatch_token(args[aid],thtt_selector_opts)) {
+
       case TT_SELECTOR_RECURSIVE:
         aid++;
         switch (thmatch_token(args[aid],thtt_bool)) {
@@ -100,10 +104,15 @@ void thselector::parse_selection (bool usid, int nargs, char ** args)
             itm.recursive = true;
             break;
           case TT_UNKNOWN_BOOL:
-            ththrow(("logical value expected -- %s", args[aid]))
+            ththrow("logical value expected -- {}", args[aid]);
         }
         break;
 
+      case TT_SELECTOR_COLOR:
+        aid++;
+        itm.m_color.parse(args[aid]);
+        break;
+
       case TT_SELECTOR_MAP_LEVEL:
         aid++;
         thparse_double(sv,dum,args[aid]);
@@ -112,9 +121,9 @@ void thselector::parse_selection (bool usid, int nargs, char ** args)
           break;
         }
         if ((sv != TT_SV_NUMBER) || (dum < 0))
-          ththrow(("invalid map level -- %s", *args))
+          ththrow("invalid map level -- {}", *args);
         if (double(int(dum)) != dum)
-          ththrow(("invalid map level -- %s", *args))
+          ththrow("invalid map level -- {}", *args);
         itm.map_level = long(dum);
         break;
 
@@ -126,14 +135,14 @@ void thselector::parse_selection (bool usid, int nargs, char ** args)
           break;
         }
         if ((sv != TT_SV_NUMBER) || (dum <= 0))
-          ththrow(("invalid chapter level -- %s", *args))
+          ththrow("invalid chapter level -- {}", *args);
         if (double(int(dum)) != dum)
-          ththrow(("invalid chapter level -- %s", *args))
+          ththrow("invalid chapter level -- {}", *args);
         itm.chapter_level = long(dum);
         break;
         
       default:
-        ththrow(("unknown option -- %s", args[aid]))
+        ththrow("unknown option -- {}", args[aid]);
     }
   }
   
@@ -147,7 +156,7 @@ class thselector_select_item {
   public:
 
   thdataobject * objp;
-  char * n1, * n2, * n3;
+  const char * n1, * n2, * n3;
   
   void clear();
   
@@ -215,7 +224,7 @@ void thselector_select_item::parse (thdataobject * op)
 
 typedef std::vector <thselector_select_item> thselector_siv;
 
-void thselector__export_survey_tree_node (FILE * cf, unsigned long level, thdataobject * optr) {
+void thselector_export_survey_tree_node (FILE * cf, unsigned long level, thdataobject * optr) {
   thsurvey * ss;
   while (optr != NULL) {
     if (optr->get_class_id() == TT_SURVEY_CMD) {
@@ -268,13 +277,13 @@ void thselector__export_survey_tree_node (FILE * cf, unsigned long level, thdata
         fprintf(cf,"\"\n");
       }
       if (ss->foptr != NULL)
-        thselector__export_survey_tree_node(cf,level+1,ss->foptr);
+        thselector_export_survey_tree_node(cf,level+1,ss->foptr);
     }
     optr = optr->nsptr;
   }
 }
 
-void thselector__prepare_map_tree_export (thdatabase * db) {
+void thselector_prepare_map_tree_export (thdatabase * db) {
 
   // prejde vsetky mapy a surveye a nastavi tmp_bool na true a tmp_ulong na 0
   thdb_object_list_type::iterator obi = db->object_list.begin();
@@ -289,7 +298,7 @@ void thselector__prepare_map_tree_export (thdatabase * db) {
   obi = db->object_list.begin();
   while (obi != db->object_list.end()) {
     if (((*obi)->fsptr != NULL) && ((*obi)->get_class_id() == TT_MAP_CMD)) {
-      mi = ((thmap*)(*obi))->first_item;
+      mi = ((thmap*)(*obi).get())->first_item;
       while (mi != NULL) {
         if (mi->type == TT_MAPITEM_NORMAL) {
           mi->object->tmp_bool = false;
@@ -302,7 +311,7 @@ void thselector__prepare_map_tree_export (thdatabase * db) {
   
 }
 
-void thselector__export_map_tree_node (FILE * cf, unsigned long level, unsigned long pass, thdataobject * optr, thmap * fmap) {
+void thselector_export_map_tree_node (FILE * cf, unsigned long level, unsigned long pass, thdataobject * optr, thmap * fmap) {
   if (optr->tmp_ulong == pass)
     return;
   if (optr->fsptr == NULL)
@@ -310,7 +319,7 @@ void thselector__export_map_tree_node (FILE * cf, unsigned long level, unsigned
   optr->tmp_ulong = pass;
   thmap * mptr = NULL;
   int subtype = 0;
-  char * types = NULL;
+  const char * types = NULL;
   switch (optr->get_class_id()) {
     case TT_MAP_CMD:
       mptr = (thmap*) optr; //id fid level
@@ -334,7 +343,7 @@ void thselector__export_map_tree_node (FILE * cf, unsigned long level, unsigned
   else
     fprintf(cf," %luX%lu %lu",fmap->id,fmap->nz,level);
 
-  fprintf(cf," %s %s@%s",optr->get_name(),optr->get_name(),optr->fsptr->full_name);
+  fprintf(cf," %s %s%s%s",optr->get_name(),optr->get_name(), (strlen(optr->fsptr->full_name) > 0 ? "@" : ""), optr->fsptr->full_name);
   if (strlen(optr->get_title()) > 0) {
     thdecode_tex(&(thdb.buff_enc), optr->get_title());
     fprintf(cf," \"%s\"",thdb.buff_enc.get_buffer());
@@ -350,7 +359,7 @@ void thselector__export_map_tree_node (FILE * cf, unsigned long level, unsigned
     mi = mptr->first_item;
     while (mi != NULL) {
       if (mi->type == TT_MAPITEM_NORMAL) {
-        thselector__export_map_tree_node(cf, level+1, pass, mi->object, mptr);
+        thselector_export_map_tree_node(cf, level+1, pass, mi->object, mptr);
       }
       mi = mi->next_item;
     }
@@ -363,35 +372,41 @@ void thselector::dump_selection_db (FILE * cf, thdatabase * db)
   // exportuje strom surveyov
   fprintf(cf,"set xth(ctrl,cp,datlist) {}\n");
   if (db->fsurveyptr != NULL) 
-    thselector__export_survey_tree_node(cf,0,db->fsurveyptr);
+    thselector_export_survey_tree_node(cf,0,db->fsurveyptr);
   fprintf(cf,"xth_cp_data_tree_create\n");
   
   // exportuje strom map po vsetkych projekciach
   fprintf(cf,"set xth(ctrl,cp,maplist) {}\n");
   // exportuje vsetky projekcie
   thdb2dprj_list::iterator prjli = db->db2d.prj_list.begin();
+	thbuffer projdir;
+	projdir.guarantee(32);
   while (prjli != db->db2d.prj_list.end()) {
+		projdir.strcpy("");
+		if ((prjli->type == TT_2DPROJ_ELEV) && (prjli->pp1 != 0.0)) {
+			sprintf(projdir.get_buffer(), "\\[%.1f\\]", prjli->pp1);
+		}
     fprintf(cf,"xth_cp_map_tree_insert projection 0 p%d {} 0",prjli->id); 
     if (strlen(prjli->index) > 0)
-      fprintf(cf," %s:%s",thmatch_string(prjli->type,thtt_2dproj),prjli->index);
+      fprintf(cf," %s%s:%s",thmatch_string(prjli->type,thtt_2dproj), projdir.get_buffer(), prjli->index);
     else
-      fprintf(cf," %s",thmatch_string(prjli->type,thtt_2dproj));
+      fprintf(cf," %s%s",thmatch_string(prjli->type,thtt_2dproj), projdir.get_buffer());
     fprintf(cf," {} {} {}\n");
     prjli++;
   }
   // exportuje vsetky mapy a scrapy
-  thselector__prepare_map_tree_export(db);
+  thselector_prepare_map_tree_export(db);
   thdb_object_list_type::iterator obi = db->object_list.begin();
   while (obi != db->object_list.end()) {
     if ((*obi)->get_class_id() == TT_MAP_CMD)
-      ((thmap*)(*obi))->nz = 0;
+      ((thmap*)(*obi).get())->nz = 0;
     obi++;
   }
 
   obi = db->object_list.begin();
   while (obi != db->object_list.end()) {
-    if (((*obi)->fsptr != NULL) && ((*obi)->get_class_id() == TT_MAP_CMD) && (((thmap*)(*obi))->tmp_bool))
-      thselector__export_map_tree_node(cf,1,(*obi)->id,(*obi),NULL);
+    if (((*obi)->fsptr != NULL) && ((*obi)->get_class_id() == TT_MAP_CMD) && (((thmap*)(*obi).get())->tmp_bool))
+      thselector_export_map_tree_node(cf,1,(*obi)->id,obi->get(),NULL);
     obi++;
   }
   
@@ -438,7 +453,7 @@ void thselector::select_db(class thdatabase * db)
   thmbuffer mbf;
   char ** nms, * nobj = NULL, * nsrv = NULL;
   int nn = 0;
-  thsurvey * objsrv;
+  //thsurvey * objsrv;
   thdataobject * objptr = NULL;
   bool to_cont = false, select_all = false, has_selected_survey = false;
   thselector_list::iterator ii = this->data.begin();
@@ -465,8 +480,8 @@ void thselector::select_db(class thdatabase * db)
     nn = mbf.get_size();
     switch (nn) {
       case 1:
-        nobj = "";
-        nsrv = *nms;
+        nobj = *nms;
+        nsrv = NULL;
         break;
       case 2:
         nobj = nms[0];
@@ -483,26 +498,11 @@ void thselector::select_db(class thdatabase * db)
       continue;
     }
     
-    if ((strlen(nsrv) == 0) || (strcmp(nsrv,"*") == 0)) {
+    if ((nsrv == NULL) && ((strlen(nobj) == 0) || (strcmp(nobj,"*") == 0))) {
       select_all = true;
     }
-    else {
-      // let's find the survey
-      try {
-        objsrv = db->get_survey(nsrv,NULL);
-        objptr = objsrv;
-      }
-      catch(...) {
-        objsrv = NULL;
-      }
-      if (objsrv == NULL) {
-        thwarning(("%s [%d] -- survey not found -- \"%s\"", 
-          ii->src_name, ii->src_ln, nsrv))
-        to_cont = false;
-      }
-    } 
     
-    if (to_cont && (nn == 2)) {
+    if ((!select_all)) {
       try {
         objptr = db->get_object(thobjectname(nobj,nsrv),NULL);
       }
@@ -554,9 +554,11 @@ void thselector::select_object(thselector_item * pitm, class thdataobject * optr
   if (pitm->unselect) {
     optr->selected = false;
     optr->selected_number = pitm->number;
+    optr->selected_color.defined = 0;
   } else {
     optr->selected = true;
     optr->selected_number = pitm->number;
+    optr->selected_color = pitm->m_color;
   }
 }
 
@@ -567,7 +569,7 @@ void thselector::select_all(thselector_item * pitm, class thdatabase * db)
   while (ii != db->object_list.end()) {
     switch((*ii)->get_class_id()) {
       case TT_SURVEY_CMD:
-        this->select_object(pitm,*ii);
+        this->select_object(pitm,ii->get());
       default:
         break;
     }
