@@ -585,7 +585,7 @@ void thsvxctrl::transcript_log_file(class thdatabase * dbp, const char * lfnm)
   while (!(clf.eof())) {
     lnum++;
     clf.getline(lnbuff,2048);
-    thlog.printf("%2d> %s\n",lnum,lnbuff);
+    thlog.printf("%2lu> %s\n",lnum,lnbuff);
     // let's scan the line
     chch = lnbuff;
     nchs = strlen(chch);
@@ -710,7 +710,7 @@ void thsvxctrl::load_err_file(class thdatabase * dbp, const char * lfnm) {
 
 //		thprintf("SCANNING: %s\n", line.c_str());
 		thsplit_args(&b, line.c_str());
-		thdb1d_traverse t, * ct;
+		thdb1d_traverse t = {}, * ct;
 		thdb1d_loop_leg l;
 		thdb1d_loop_leg * cl;
 		thdb1d_tree_node * cn;
