@@ -968,17 +968,27 @@ thvec2 thlinzoomtrans::forward(thvec2 src) {
       dlt = this->m_line_to.eval(src);
       if (dlf < 0.0) {
         vec = src - this->m_from;
-        vo = vec.orientation() - this->m_orient_from;
-        if (vo < 0.0) vo += 360.0;
-        vo /= 180.0;
+        vo = atan2(sin((vec.orientation() - this->m_orient_from) / 180.0 * THPI), cos((vec.orientation() - this->m_orient_from) / 180.0 * THPI)) / THPI;
+        if (vo < 0.0) {
+        	if (vo > -.5)
+        		vo = 0.0;
+        	else
+        		vo = 1.0;
+        }
         vec *= (vo * this->m_fl + (1.0 - vo) * this->m_fr);
+        //thprintf("VF: %.8f (%.8f, %.8f)\n", vo, (this->m_from + vec).m_x, (this->m_from + vec).m_y);
         return this->m_from + vec;
       } else if (dlt < 0.0) {
         vec = src - this->m_to;
-        vo = this->m_orient_to - vec.orientation() - 180.0;
-        if (vo < 0.0) vo += 360.0;
-        vo /= 180.0;
+        vo = atan2(sin((this->m_orient_to - vec.orientation() - 180.0) / 180.0 * THPI), cos((this->m_orient_to - vec.orientation() - 180.0) / 180.0 * THPI)) / THPI;
+        if (vo < 0.0) {
+        	if (vo > -.5)
+        		vo = 0.0;
+        	else
+        		vo = 1.0;
+        }
         vec *= (vo * this->m_tl + (1.0 - vo) * this->m_tr);
+        //thprintf("VT: %.8f (%.8f, %.8f)\n", vo, (this->m_to + vec).m_x, (this->m_to + vec).m_y);
         return this->m_to + vec;
       } else {
         thvec2 lp;
@@ -993,6 +1003,7 @@ thvec2 thlinzoomtrans::forward(thvec2 src) {
         } else {
           vec *= (vo * this->m_tr + (1.0 - vo) * this->m_fr);
         }
+        //thprintf("VI: %.8f (%.8f, %.8f)\n", vo, (lp + vec).m_x, (lp + vec).m_y);
         return lp + vec;
       }
     }
