@@ -25,6 +25,17 @@
 ## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ## -------------------------------------------------------------------- 
 
+proc xth_mm2px {mm} {
+  global xth
+  set mmw [winfo screenmmwidth $xth(gui,main)]		
+  set w [winfo screenwidth $xth(gui,main)]
+  if {$mmw != 0} {
+    return [expr int(double($mm) * double($w) / double($mmw))]
+  } else {
+    return [expr int(4 * $mm)]
+  }
+}
+
 
 proc xth_app_move_panel {aname xx} {
   global xth
@@ -85,25 +96,32 @@ proc xth_app_create {aname title} {
   pack $aw.af -expand yes -fill both
   pack $aw.sf -side bottom -fill x
 
+  set ww [winfo width $xth(gui,main)]
+  if {$ww == 1} {
+    set ww 1024
+  }
+	
   set fr $xth(app,$aname,relw)
   set minfr $xth(app,$aname,wmwd)
-  set lrhny [expr [winfo height $xth(gui,main)] - 64]
   if {$fr < $minfr} {
     set fr $minfr
-  } elseif {$fr > ([winfo width $xth(gui,main)] - $xth(app,$aname,wmwd))} {
-    set fr [expr {([winfo width $xth(gui,main)] - $xth(app,$aname,wmwd))}]
+  } elseif {$fr > ($ww - $xth(app,$aname,wmwd))} {
+    set fr [expr {($ww - $xth(app,$aname,wmwd))}]
   }
   set xth(app,$aname,relw) $fr
-  set fr [expr 1.0 - $fr / double([winfo width $xth(gui,main)])]
+  set fr [expr 1.0 - $fr / double($ww)]
+  set bw [xth_mm2px 1.5]
+  set bwr [expr double($bw) / double($ww)]
+      	
   
   if {$xth(app,$aname,wpsw) == 1} {
     place $aw.af.apps -relx 0 -rely 0 -relheight 1 -relwidth $fr
-    place $aw.af.ctrl -relx $fr -rely 0 -relheight 1 -relwidth [expr 1.0 - $fr]
-    place $aw.af.lrhn -relx $fr -y $lrhny -width 8 -height 8 -anchor center
+    place $aw.af.ctrl -relx $fr -rely 0 -relheight 1 -relwidth [expr 1.0 - $fr - $bwr]
+    place $aw.af.lrhn -relx $fr -rely 0 -relheight 1 -width $bw -anchor n
   } else {
     place $aw.af.ctrl -relx 0 -rely 0 -relheight 1 -relwidth $fr
-    place $aw.af.apps -relx $fr -rely 0 -relheight 1 -relwidth [expr 1.0 - $fr]
-    place $aw.af.lrhn -relx $fr -y $lrhny -width 8 -height 8 -anchor center
+    place $aw.af.apps -relx $fr -rely 0 -relheight 1 -relwidth [expr 1.0 - $fr - $bwr]
+    place $aw.af.lrhn -relx $fr -rely 0 -relheight 1 -width $bw -anchor n
   }
   
   xth_ctrl_create $aname
@@ -132,17 +150,18 @@ proc xth_app_place {aname} {
   }
   set xth(app,$aname,relw) $fr
   set fr [expr 1.0 - $fr / double([winfo width $xth(gui,main)])]
+  set bw [xth_mm2px 1.72]
+  set bwr [expr double($bw) / double([winfo width $xth(gui,main)])]
+  #puts [expr $fr + $bwr]
 
-  set lrhny [expr [winfo height $xth(gui,main)] - 64]
-  
   if {$xth(app,$aname,wpsw) == 1} {
     place configure $aw.af.apps -relx 0 -relwidth $fr
-    place configure $aw.af.ctrl -relx $fr -relwidth [expr 1.0 - $fr]
-    place configure $aw.af.lrhn -relx $fr -y $lrhny
+    place configure $aw.af.ctrl -relx [expr $fr + $bwr] -relwidth [expr 1.0 - $fr - $bwr]
+    place configure $aw.af.lrhn -relx [expr $fr + 0.5 * $bwr]
   } else {
     place configure $aw.af.apps -relx [expr 1.0 - $fr] -relwidth $fr
-    place configure $aw.af.ctrl -relx 0 -relwidth [expr 1.0 - $fr]
-    place configure $aw.af.lrhn -relx [expr 1.0 - $fr] -y $lrhny
+    place configure $aw.af.ctrl -relx 0 -relwidth [expr 1.0 - $fr - $bwr]
+    place configure $aw.af.lrhn -relx [expr 1.0 - $fr - 0.5 * $bwr]
   }
 
   xth_ctrl_reshape $aname   
@@ -201,6 +220,9 @@ proc xth_app_finish {} {
   $m add command -label [mc "Switch panels"] -underline 1 \
       -command "xth_app_switch" -font $xth(gui,lfont)
   $m add separator
+
+  $m add command -label [mc "Toggle toolbar"] -underline 1 \
+      -command "xth_tools_toggle" -font $xth(gui,lfont)
   
   menu $m.kbes -tearoff 0
   set encnames [encoding names]
@@ -220,6 +242,7 @@ proc xth_app_finish {} {
     menu $dm -tearoff 0
   
     $dm add command -label "Refresh procs" -underline 0 -command {
+      set idir $xth(gui,initdir)
       source global.tcl
       source sbar.tcl
       source cp_procs.tcl
@@ -228,8 +251,10 @@ proc xth_app_finish {} {
       source me_cmds.tcl
       source me_cmds2.tcl
       source me_ss.tcl
+      source te_import.tcl
+      source me_import.tcl
       source bac.tcl
-      source mv_procs.tcl
+      set xth(gui,initdir) $idir
     } -font $xth(gui,lfont)
     $dm add command -label "Screen dump" -underline 0 -command {
       after 5000 {xwd -out screendump -frame}
@@ -252,12 +277,16 @@ proc xth_app_finish {} {
   bind $xth(gui,main) <$xth(kb_control)-Key-l> xth_app_control_l 
   bind $xth(gui,main) <$xth(kb_control)-Key-d> xth_app_control_d
   bind $xth(gui,main) <$xth(kb_control)-Key-a> xth_app_control_a
+  bind $xth(gui,main) <$xth(kb_control)-Key-D> xth_app_control_shift_d
   bind $xth(gui,main) <Prior> xth_app_pgup
   bind $xth(gui,main) <Next> xth_app_pgdn
   bind $xth(gui,main) <Shift-Prior> xth_app_shift_pgup
   bind $xth(gui,main) <Shift-Next> xth_app_shift_pgdn
   bind $xth(gui,main) <Key-Escape> xth_app_escape 
   bind $xth(gui,main) <F9> xth_app_make
+  bind $xth(gui,main) <$xth(kb_control)-Up> xth_app_control_up
+  bind $xth(gui,main) <$xth(kb_control)-Down> xth_app_control_down
+  
   foreach aname $xth(app,list) {
     $xth($aname,menu) add cascade -label [mc "Window"] -menu $m -underline 0 \
       -font $xth(gui,lfont)
@@ -310,13 +339,12 @@ proc xth_app_control_o {} {
 
   # puts $xth(app,active)  
   switch $xth(app,active) {
-    te  {xth_te_open_file 1 {} 1}
+    te  {xth_te_open_file 1 {} 0}
     me  {xth_me_open_file 1 {} 1}
     cp  {
       set xth(cp,updcf) 0
       xth_cp_open_file {}
     }
-    mv {xth_mv_open_file {}}
   }
 }  
 
@@ -329,7 +357,6 @@ proc xth_app_pgup {} {
       $xth(gui,main).me.af.ctrl.c yview scroll -1 pages
     }
     cp  {}
-    mv {}
   }
 }
 
@@ -341,7 +368,6 @@ proc xth_app_pgdn {} {
       $xth(gui,main).me.af.ctrl.c yview scroll 1 pages
     }
     cp  {}
-    mv {}
   }
 }
 
@@ -353,7 +379,6 @@ proc xth_app_shift_pgup {} {
       $xth(ctrl,me,cmds).cl.l yview scroll -1 pages
     }
     cp  {}
-    mv {}
   }
 }
 
@@ -366,7 +391,6 @@ proc xth_app_shift_pgdn {} {
       $xth(ctrl,me,cmds).cl.l yview scroll 1 pages
     }
     cp  {}
-    mv {}
   }
 }
 
@@ -384,7 +408,6 @@ proc xth_app_control_a {} {
       xth_ctrl_maximize me ac
     }
     cp  {}
-    mv {}
   }
 }  
 
@@ -403,7 +426,6 @@ proc xth_app_control_r {} {
     }
     
     cp  {}
-    mv {xth_mv_reload_file}
   }
 }  
 
@@ -466,6 +488,12 @@ proc xth_app_control_d {} {
   }
 }  
 
+proc xth_app_control_shift_d {} {
+  global xth
+  switch $xth(app,active) {
+    me  {xth_me_cmds_delete_linept {} {}}
+  }
+}  
 
 proc xth_app_control_l {} {
   global xth
@@ -487,6 +515,26 @@ proc xth_app_escape {} {
   }
 }
 
+proc xth_app_control_up {} {
+  global xth
+  switch $xth(app,active) {
+    me {  
+			if {$xth(me,zoom)*2 > 800} return;
+      xth_me_area_zoom_to [ expr {$xth(me,zoom) * 2} ]
+    }  
+  }
+}
+
+proc xth_app_control_down {} {
+  global xth
+  switch $xth(app,active) {
+    me {  
+			if {$xth(me,zoom)*2 < 12} return;
+			xth_me_area_zoom_to [ expr {$xth(me,zoom) / 2} ]
+    }  
+  }
+}
+
 
 proc xth_app_show {aname} {
 
@@ -504,7 +552,6 @@ proc xth_app_show {aname} {
   pack $xth(gui,$aname) -expand yes -fill both
 
   xth_app_title $aname
-
   $xth(gui,main) configure -menu $xth($aname,menu)
   
   regexp {([0-9]+)x([0-9]+)} [winfo geometry $xth(gui,main)] geom xsize ysize
@@ -524,6 +571,28 @@ proc xth_app_show {aname} {
 
   update idletasks  
   xth_ctrl_reshape $aname
+  
+  # toolbar changes
+  # disable map editor tools
+  catch {
+    pack forget $xth(gui,tb).edif
+    pack forget $xth(gui,tb).self
+  }
+  switch $aname {
+    me {
+      # enable map editor tools
+      pack $xth(gui,tb).self -side right -anchor nw
+      pack $xth(gui,tb).edif -side right
+      xth_me_unredo_update
+    }
+    default {
+      $xth(gui,tb).undob configure -state normal
+      $xth(gui,tb).redob configure -state normal
+    }
+  }
+
+  update idletasks
+  wm geometry $xth(gui,main) [wm geometry $xth(gui,main)]
 }
 
 
@@ -531,6 +600,10 @@ proc xth_exit {} {
 
   global xth
 
+  catch {
+    registry set $xth(win32registry) XTherionDir $xth(gui,initdir)
+  }
+  
   # save all open text editor files
   if {![info exists xth(te,flist)]} {
     set xth(te,flist) {}
@@ -567,7 +640,7 @@ proc xth_app_maximize {} {
   set shg [winfo screenheight $xth(gui,main)]
   wm geometry $xth(gui,main) [format "%dx%d+0+0" $swd $shg]
   update idletasks
-  regexp {([0-9]+)x([0-9]+)\+([0-9]+)\+([0-9]+)} [winfo geometry $xth(gui,main)] geom xsize ysize xshft yshft
+  regexp {([0-9]+)x([0-9]+)\+(-?[0-9]+)\+(-?[0-9]+)} [winfo geometry $xth(gui,main)] geom xsize ysize xshft yshft
   wm geometry $xth(gui,main) [format "%dx%d+0+0" [expr $swd - $xshft] [expr $shg - $yshft]]
   update idletasks
 }
@@ -586,7 +659,7 @@ proc xth_app_normalize {} {
   set tpy [expr int(0.5 * ([winfo screenheight $xth(gui,main)] - $thg))]
   wm geometry $xth(gui,main) [format "%dx%d+%d+%d" $twd $thg $tpx $tpy]
   update idletasks
-  regexp {([0-9]+)x([0-9]+)\+([0-9]+)\+([0-9]+)} [winfo geometry $xth(gui,main)] geom xsize ysize xshft yshft
+  regexp {([0-9]+)x([0-9]+)\+(-?[0-9]+)\+(-?[0-9]+)} [winfo geometry $xth(gui,main)] geom xsize ysize xshft yshft
   wm geometry $xth(gui,main) [format "%dx%d+%d+%d" [expr $twd - $xshft + $tpx] \
     [expr $thg - $yshft + $tpy] $tpx $tpy]
   update idletasks
@@ -687,4 +760,77 @@ proc xth_app_autosave {} {
   xth_app_autosave_schedule
 }
 
+proc tk_textPaste w {
+  global tcl_platform
+  if {![catch {::tk::GetSelection $w CLIPBOARD} sel]} then {
+    set oldSeparator [$w cget -autoseparators]
+    if {$oldSeparator} then {
+      $w configure -autoseparators 0
+      $w edit separator
+    }
+    foreach {to from} [lreverse [$w tag ranges sel]] {
+      $w delete $from $to
+    }
+    $w insert insert $sel
+    if {$oldSeparator} then {
+      $w edit separator
+      $w configure -autoseparators 1
+    }
+  }
+}
 
+proc tk_textDelete w {
+  global tcl_platform
+  set text_sel_removed 0
+  if {![catch {::tk::GetSelection $w CLIPBOARD} sel]} then {
+    set oldSeparator [$w cget -autoseparators]
+    if {$oldSeparator} then {
+      $w configure -autoseparators 0
+      $w edit separator
+    }
+    foreach {to from} [lreverse [$w tag ranges sel]] {
+      set text_sel_removed 1
+      $w delete $from $to
+    }
+    if {$oldSeparator} then {
+      $w edit separator
+      $w configure -autoseparators 1
+    }
+  }
+  if {!$text_sel_removed} then {
+    $w delete insert
+    $w see insert
+  }
+}
+
+proc tk_entryPaste w {
+  global tcl_platform
+  if {![catch {::tk::GetSelection $w CLIPBOARD} sel]} then {
+    if {[$w selection present]} then { $w delete sel.first sel.last }
+    $w insert insert $sel
+  }
+}
+
+proc tk_entryDelete w {
+  global tcl_platform
+  if {[$w selection present]} then {
+    $w delete sel.first sel.last
+  } else {
+    $w delete insert
+  }
+}
+
+proc xth_app_text_copy_paste_binds {w {cond {if {1}}}} {
+  global xth
+  bind $w <$xth(kb_control)-Key-x> "$cond {tk_textCut %W}"
+  bind $w <$xth(kb_control)-Key-c> "$cond {tk_textCopy %W}"
+  bind $w <$xth(kb_control)-Key-v> "$cond {tk_textPaste %W}"
+  bind $w <$xth(kb_control)-Key-z> "$cond {catch {%W edit undo}}"
+  bind $w <$xth(kb_control)-Key-y> "$cond {catch {%W edit redo}}"
+  if {$xth(gui,bindinsdel)} {
+    bind $w <Delete>                      "$cond {tk_textDelete %W}"
+    bind $w <Shift-Key-Delete>            "$cond {tk_textCut %W}"
+    bind $w <$xth(kb_control)-Key-Insert> "$cond {tk_textCopy %W}"
+    bind $w <Shift-Key-Insert>            "$cond {tk_textPaste %W}"
+  }
+}
